@@ -23,6 +23,13 @@
 ** See the GNU General Public License for more details.
 **
 ** $Log: acctproc.c,v $
+** Revision 1.26  2008/03/06 08:37:25  gerlof
+** Register/show ppid of a process.
+**
+** Revision 1.25  2008/01/14 09:22:23  gerlof
+** Support for environment variable ATOPACCT to specify the name of a
+** particular process accouting file.
+**
 ** Revision 1.24  2007/08/17 08:50:26  gerlof
 ** Verify if private accounting used before switching off accounting.
 **
@@ -100,7 +107,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: acctproc.c,v 1.24 2007/08/17 08:50:26 gerlof Exp $";
+static const char rcsid[] = "$Id: acctproc.c,v 1.26 2008/03/06 08:37:25 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -121,6 +128,7 @@ static const char rcsid[] = "$Id: acctproc.c,v 1.24 2007/08/17 08:50:26 gerlof E
 
 #define	ACCTDIR		"/tmp/atop.d"
 #define	ACCTFILE	"atop.acct"
+#define	ACCTENV		"ATOPACCT"
 
 static	char 	acctatop;	/* boolean: atop's own accounting busy ?  */
 static	off_t	acctsize;	/* previous size of account file	  */
@@ -136,7 +144,6 @@ struct pacctadm {
 	char		*name;
 	struct stat	stat;
 } pacctadm[] = {
-	{ "/var/log/account/pacct",	{0, }, },
 	{ "/var/log/pacct",		{0, }, },
 	{ "/var/account/pacct",		{0, }, }
 };
@@ -175,9 +182,58 @@ acctswon(void)
 	static ushort 		vals[] = {1, SEMTOTAL};
 	union {ushort *array;}	arg = {vals};
 	struct stat		statbuf;
+	char			*ep;
 
 	/*
-	** first check if process-accounting is already switched on 
+	** when a particular environment variable is present, atop should
+	** use a specific accounting-file (as defined by the environment
+	** variable) or should use no process accounting at all (when
+	** contents of environment variable is empty)
+	*/
+	if ( (ep = getenv(ACCTENV)) )
+	{
+		/*
+		** environment variable exists
+		*/
+		if (*ep)
+		{
+			/*
+			** open active account file with the specified name
+			*/
+			seteuid( getuid() );	/* drop setuid-root privs */
+
+			if ( (acctfd = open(ep, O_RDONLY) ) == -1)
+			{
+				perror("open account-file");
+				fprintf(stderr,
+			                "warning: no process exit detection\n");
+				sleep(3);
+				return 0;
+			}
+
+			if ( !acctvers(acctfd) )
+			{
+				fprintf(stderr,
+               				"warning: no process exit detection\n");
+				sleep(3);
+				(void) close(acctfd);
+				return 0;
+			}
+
+			supportflags |= ACCTACTIVE;
+			return 1;
+		}
+		else
+		{
+			/*
+			** no contents
+			*/
+			return 0;
+		}
+	}
+
+	/*
+	** check if process-accounting is already switched on 
 	** for one of the standard accounting-files; in that case we
 	** open this file and get our info from here ....
 	*/
@@ -228,6 +284,7 @@ acctswon(void)
 	                				"warning: no process "
 						        "exit detection\n");
 						sleep(3);
+						(void) close(acctfd);
 						return 0;
 					}
 
@@ -569,6 +626,7 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			*/
 			api->gen.state  = 'E';
 			api->gen.nthr   = 1;
+			api->gen.ppid   = 0;
 			api->gen.excode = acctrec.ac_exitcode;
 			api->gen.ruid   = acctrec.ac_uid16;
 			api->gen.rgid   = acctrec.ac_gid16;
@@ -591,6 +649,7 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			*/
 			api->gen.state  = 'E';
 			api->gen.pid    = acctrec_v3.ac_pid;
+			api->gen.ppid   = acctrec_v3.ac_ppid;
 			api->gen.nthr   = 1;
 			api->gen.excode = acctrec_v3.ac_exitcode;
 			api->gen.ruid   = acctrec_v3.ac_uid;
@@ -614,6 +673,7 @@ acctphotoproc(struct pstat *accproc, int nrprocs)
 			*/
 			api->gen.state  = 'E';
 			api->gen.pid    = acctrec_atop.ac_pid;
+			api->gen.ppid   = acctrec_atop.ac_ppid;
 			api->gen.nthr   = 1;
 			api->gen.excode = acctrec_atop.ac_exitcode;
 			api->gen.ruid   = acctrec_atop.ac_uid;
