@@ -39,6 +39,12 @@ typedef	long long	count_t;
 struct pstat;
 struct sstat;
 
+/* 
+** miscellaneous flags
+*/
+#define RRBOOT  0x0001
+#define RRLAST  0x0002
+
 struct visualize {
 	char	(*show_samp)  (time_t, int, struct sstat *, struct pstat *,
 				int, int, int, int, char);
@@ -84,8 +90,8 @@ extern int		almostcrit;
 */
 #define	ACCTACTIVE	0x00000001
 #define	PATCHSTAT	0x00000002
-#define	PATCHACCT	0x00000003
 #define	IOSTAT		0x00000004
+#define	PATCHACCT	0x00000008
 
 /*
 ** structure containing the start-addresses of functions for visualization
