@@ -30,6 +30,15 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: deviate.c,v $
+** Revision 1.39  2008/02/25 14:51:18  gerlof
+** Experimental code for HTTP-statistics.
+**
+** Revision 1.38  2008/01/07 11:33:43  gerlof
+** Cosmetic changes.
+**
+** Revision 1.37  2008/01/07 10:17:24  gerlof
+** Implement possibility to make summaries.
+**
 ** Revision 1.36  2007/11/05 12:13:16  gerlof
 ** Match processes not only on pid, but also on start time.
 **
@@ -141,7 +150,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.36 2007/11/05 12:13:16 gerlof Exp $";
+static const char rcsid[] = "$Id: deviate.c,v 1.39 2008/02/25 14:51:18 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -749,7 +758,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 
 		dev->intf.intf[i].speed 	= pre->intf.intf[i].speed;
 		dev->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
-	
+
 		/*
 		** save interface properties for next interval
 		*/
@@ -806,6 +815,231 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 #endif
 }
 
+/*
+** add the values of a new sample to a structure holding the totals
+** for the indicated category (c=cpu, m=memory, d=disk, n=network).
+*/
+void
+totalsyst(char category, struct sstat *new, struct sstat *tot)
+{
+	register int	i;
+	count_t		*ctot, *cnew;
+
+	switch (category)
+	{
+	   case 'c':	/* accumulate cpu-related counters */
+		tot->cpu.nrcpu      = new->cpu.nrcpu;
+		tot->cpu.devint    += new->cpu.devint;
+		tot->cpu.csw       += new->cpu.csw;
+		tot->cpu.nprocs    += new->cpu.nprocs;
+
+		tot->cpu.all.stime += new->cpu.all.stime;
+		tot->cpu.all.utime += new->cpu.all.utime;
+		tot->cpu.all.ntime += new->cpu.all.ntime;
+		tot->cpu.all.itime += new->cpu.all.itime;
+		tot->cpu.all.wtime += new->cpu.all.wtime;
+		tot->cpu.all.Itime += new->cpu.all.Itime;
+		tot->cpu.all.Stime += new->cpu.all.Stime;
+		tot->cpu.all.steal += new->cpu.all.steal;
+
+		if (new->cpu.nrcpu == 1)
+		{
+			tot->cpu.cpu[0] = tot->cpu.all;
+		}
+		else
+		{
+			for (i=0; i < new->cpu.nrcpu; i++)
+			{
+				tot->cpu.cpu[i].cpunr  = new->cpu.cpu[i].cpunr;
+				tot->cpu.cpu[i].stime += new->cpu.cpu[i].stime;
+				tot->cpu.cpu[i].utime += new->cpu.cpu[i].utime;
+				tot->cpu.cpu[i].ntime += new->cpu.cpu[i].ntime;
+				tot->cpu.cpu[i].itime += new->cpu.cpu[i].itime;
+				tot->cpu.cpu[i].wtime += new->cpu.cpu[i].wtime;
+				tot->cpu.cpu[i].Itime += new->cpu.cpu[i].Itime;
+				tot->cpu.cpu[i].Stime += new->cpu.cpu[i].Stime;
+				tot->cpu.cpu[i].steal += new->cpu.cpu[i].steal;
+			}
+		}
+
+		tot->cpu.lavg1	 = new->cpu.lavg1;
+		tot->cpu.lavg5	 = new->cpu.lavg5;
+		tot->cpu.lavg15	 = new->cpu.lavg15;
+		break;
+
+	   case 'm':	/* accumulate memory-related counters */
+		tot->mem.physmem	 = new->mem.physmem;
+		tot->mem.freemem	 = new->mem.freemem;
+		tot->mem.buffermem	 = new->mem.buffermem;
+		tot->mem.slabmem	 = new->mem.slabmem;
+		tot->mem.committed	 = new->mem.committed;
+		tot->mem.commitlim	 = new->mem.commitlim;
+		tot->mem.cachemem	 = new->mem.cachemem;
+		tot->mem.totswap	 = new->mem.totswap;
+		tot->mem.freeswap	 = new->mem.freeswap;
+
+		tot->mem.swouts		+= new->mem.swouts;
+		tot->mem.swins		+= new->mem.swins;
+		tot->mem.pgscans	+= new->mem.pgscans;
+		tot->mem.allocstall	+= new->mem.allocstall;
+		break;
+
+	   case 'n':	/* accumulate network-related counters */
+		/*
+		** structures with network-related counters are considered
+		** as tabels of frequency-counters that will be accumulated;
+		** values that do not represent a frequency are corrected
+		** afterwards
+		*/
+		for (ctot = (count_t *)&tot->net.ipv4,
+		     cnew = (count_t *)&new->net.ipv4, i = 0;
+			i < (sizeof tot->net.ipv4 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+
+		tot->net.ipv4.Forwarding = new->net.ipv4.Forwarding;
+		tot->net.ipv4.DefaultTTL = new->net.ipv4.DefaultTTL;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.icmpv4,
+		     cnew = (count_t *)&new->net.icmpv4, i = 0;
+			i < (sizeof tot->net.icmpv4 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.udpv4,
+		     cnew = (count_t *)&new->net.udpv4, i = 0;
+			i < (sizeof tot->net.udpv4 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.ipv6,
+		     cnew = (count_t *)&new->net.ipv6, i = 0;
+			i < (sizeof tot->net.ipv6 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.icmpv6,
+		     cnew = (count_t *)&new->net.icmpv6, i = 0;
+			i < (sizeof tot->net.icmpv6 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.udpv6,
+		     cnew = (count_t *)&new->net.udpv6, i = 0;
+			i < (sizeof tot->net.udpv6 / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+			/* ------------- */
+	
+		for (ctot = (count_t *)&tot->net.tcp,
+		     cnew = (count_t *)&new->net.tcp, i = 0;
+			i < (sizeof tot->net.tcp / sizeof(count_t));
+		     		ctot++, cnew++, i++)
+					*ctot += *cnew;
+	
+		tot->net.tcp.RtoAlgorithm = new->net.tcp.RtoAlgorithm;
+		tot->net.tcp.RtoMin       = new->net.tcp.RtoMin;
+		tot->net.tcp.RtoMax       = new->net.tcp.RtoMax;
+		tot->net.tcp.MaxConn      = new->net.tcp.MaxConn;
+		tot->net.tcp.CurrEstab    = new->net.tcp.CurrEstab;
+	
+		for (i=0; new->intf.intf[i].name[0]; i++)
+		{
+			/*
+			** check if an interface has been added or removed;
+			** in that case, zero all counters
+			*/
+			if (strcmp(new->intf.intf[i].name,
+			           tot->intf.intf[i].name) != 0)
+			{
+				tot->intf.intf[i].rbyte    = 0;
+				tot->intf.intf[i].rpack    = 0;
+				tot->intf.intf[i].rerrs    = 0;
+				tot->intf.intf[i].rdrop    = 0;
+				tot->intf.intf[i].rfifo    = 0;
+				tot->intf.intf[i].rframe   = 0;
+				tot->intf.intf[i].rcompr   = 0;
+				tot->intf.intf[i].rmultic  = 0;
+	
+				tot->intf.intf[i].sbyte    = 0;
+				tot->intf.intf[i].spack    = 0;
+				tot->intf.intf[i].serrs    = 0;
+				tot->intf.intf[i].sdrop    = 0;
+				tot->intf.intf[i].sfifo    = 0;
+				tot->intf.intf[i].scollis  = 0;
+				tot->intf.intf[i].scarrier = 0;
+				tot->intf.intf[i].scompr   = 0;
+			}
+	
+			/*
+			** accumulate counters for this sample
+			*/
+			strcpy(tot->intf.intf[i].name, new->intf.intf[i].name);
+	
+			tot->intf.intf[i].rbyte   += new->intf.intf[i].rbyte;
+			tot->intf.intf[i].rpack   += new->intf.intf[i].rpack;
+			tot->intf.intf[i].rerrs   += new->intf.intf[i].rerrs;
+			tot->intf.intf[i].rdrop   += new->intf.intf[i].rdrop;
+			tot->intf.intf[i].rfifo   += new->intf.intf[i].rfifo;
+			tot->intf.intf[i].rframe  += new->intf.intf[i].rframe;
+			tot->intf.intf[i].rcompr  += new->intf.intf[i].rcompr;
+			tot->intf.intf[i].rmultic += new->intf.intf[i].rmultic;
+	
+			tot->intf.intf[i].sbyte   += new->intf.intf[i].sbyte;
+			tot->intf.intf[i].spack   += new->intf.intf[i].spack;
+			tot->intf.intf[i].serrs   += new->intf.intf[i].serrs;
+			tot->intf.intf[i].sdrop   += new->intf.intf[i].sdrop;
+			tot->intf.intf[i].sfifo   += new->intf.intf[i].sfifo;
+			tot->intf.intf[i].scollis += new->intf.intf[i].scollis;
+			tot->intf.intf[i].scarrier+= new->intf.intf[i].scarrier;
+			tot->intf.intf[i].scompr  += new->intf.intf[i].scompr;
+	
+			tot->intf.intf[i].speed    = new->intf.intf[i].speed;
+			tot->intf.intf[i].duplex   = new->intf.intf[i].duplex;
+		}
+	
+		tot->intf.intf[i].name[0] = '\0';
+		tot->intf.nrintf          = i;
+
+#if		HTTPSTATS
+		tot->www.accesses  += new->www.accesses;
+		tot->www.totkbytes += new->www.totkbytes;
+		tot->www.bworkers   = new->www.bworkers;
+		tot->www.iworkers   = new->www.iworkers;
+#endif
+		break;
+
+	   case 'd':	/* accumulate disk-related counters */
+		for (i=0; new->xdsk.xdsk[i].name[0]; i++)
+		{
+			strcpy(tot->xdsk.xdsk[i].name, new->xdsk.xdsk[i].name);
+	
+			tot->xdsk.xdsk[i].nread  += new->xdsk.xdsk[i].nread;
+			tot->xdsk.xdsk[i].nwrite += new->xdsk.xdsk[i].nwrite;
+			tot->xdsk.xdsk[i].nrsect += new->xdsk.xdsk[i].nrsect;
+			tot->xdsk.xdsk[i].nwsect += new->xdsk.xdsk[i].nwsect;
+			tot->xdsk.xdsk[i].io_ms  += new->xdsk.xdsk[i].io_ms;
+			tot->xdsk.xdsk[i].avque  += new->xdsk.xdsk[i].avque;
+		}
+	
+		tot->xdsk.xdsk[i].name[0] = '\0';
+		tot->xdsk.nrxdsk = i;
+		break;
+	}
+}
+
+
 /*
 ** Generic function to subtract two counters taking into 
 ** account the possibility of overflow of a 32-bit kernel-counter.
