@@ -26,8 +26,21 @@
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
 **
-** $Id: parseable.c,v 1.3 2007/08/16 12:00:11 gerlof Exp $
+** $Id: parseable.c,v 1.7 2008/03/06 09:08:29 gerlof Exp $
 ** $Log: parseable.c,v $
+** Revision 1.7  2008/03/06 09:08:29  gerlof
+** Bug-solution regarding parseable output of PPID.
+**
+** Revision 1.6  2008/03/06 08:38:03  gerlof
+** Register/show ppid of a process.
+**
+** Revision 1.5  2008/01/18 08:03:40  gerlof
+** Show information about the number of threads in state 'running',
+** 'interruptible sleeping' and 'non-interruptible sleeping'.
+**
+** Revision 1.4  2007/12/11 13:33:12  gerlof
+** Cosmetic change.
+**
 ** Revision 1.3  2007/08/16 12:00:11  gerlof
 ** Add support for atopsar reporting.
 ** Concerns networking-counters that have been changed.
@@ -175,7 +188,7 @@ char
 parseout(time_t curtime, int numsecs,
 	 struct sstat *ss, struct pstat *ps,
 	 int nlist, int npresent, int nzombie,
-	 int nexit, char bootflag)
+	 int nexit, char flag)
 {
 	register int	i;
 	char		datestr[32], timestr[32], header[256];
@@ -371,18 +384,22 @@ print_PRG(char *hp, struct sstat *ss, struct pstat *ps, int nact)
 
 	for (i=0; i < nact; i++, ps++)
 	{
-		printf("%s %d (%s) %c %d %d %d %d %d %ld (%s)\n",
+		printf("%s %d (%s) %c %d %d %d %d %d %ld (%s) %d %d %d %d\n",
 				hp,
 				ps->gen.pid,
 				ps->gen.name,
 				ps->gen.state,
 				ps->gen.ruid,
 				ps->gen.rgid,
-				ps->gen.tgid,
+				ps->gen.pid,	/* was TGID formerly */
 				ps->gen.nthr,
 				ps->gen.excode,
 				ps->gen.btime,
-				ps->gen.cmdline);
+				ps->gen.cmdline,
+				ps->gen.ppid,
+				ps->gen.nthrrun,
+				ps->gen.nthrslpi,
+				ps->gen.nthrslpu);
 	}
 }
 
