@@ -31,6 +31,12 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photoproc.c,v $
+** Revision 1.31  2008/03/06 08:38:14  gerlof
+** Register/show ppid of a process.
+**
+** Revision 1.30  2008/01/18 07:36:29  gerlof
+** Gather information about the state of the individual threads.
+**
 ** Revision 1.29  2007/11/05 12:26:10  gerlof
 ** Detect disappearing /proc/stat file  when process exits
 ** (credits: Rene Rebe).
@@ -124,7 +130,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photoproc.c,v 1.29 2007/11/05 12:26:10 gerlof Exp $";
+static const char rcsid[] = "$Id: photoproc.c,v 1.31 2008/03/06 08:38:14 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -137,7 +143,7 @@ static const char rcsid[] = "$Id: photoproc.c,v 1.29 2007/11/05 12:26:10 gerlof
 #include "atop.h"
 #include "photoproc.h"
 
-#define	SCANSTAT 	"%c   %*d  %*d  %*d  %*d  %*d  "	\
+#define	SCANSTAT 	"%c   %d   %*d  %*d  %*d  %*d  "	\
 			"%*d  %lld %*d  %lld %*d  %lld "	\
 			"%lld %*d  %*d  %d   %d   %*d  "	\
 			"%*d  %lld %lld %lld %*d  %lld "	\
@@ -289,7 +295,7 @@ fillproc(struct pstat *curproc)
 	int		nr;
 	char		line[4096], *cmdhead, *cmdtail;
 	char		command[64], state;
-	int		pid, tgid, ruid, rgid, prio, policy, rtprio, nice,
+	int		pid, ppid, ruid, rgid, prio, policy, rtprio, nice,
 			curcpu, nthreads, sleepavg;
 	count_t		utime, stime, starttime;
 	count_t		minflt, majflt, size, rss, nswap,
@@ -326,7 +332,7 @@ fillproc(struct pstat *curproc)
 	rtprio = policy = 0;
 
 	nr = sscanf(cmdtail+2, SCANSTAT,
-		&state,    &minflt,    &majflt,
+		&state,    &ppid,      &minflt,    &majflt,
 		&utime,    &stime,     &prio,      &nice,    &starttime,
 		&size,     &rss,       &startcode, &endcode, &nswap,
 		&curcpu,   &rtprio,    &policy);
@@ -351,7 +357,6 @@ fillproc(struct pstat *curproc)
 	if ( (fp = fopen("status", "r")) == NULL)
 		return 0;
 
-	tgid     = pid;		/* for compat with 2.4 */
 	nthreads = 1;		/* for compat with 2.4 */
 	sleepavg = 0;		/* for compat with 2.4 */
 
@@ -363,12 +368,6 @@ fillproc(struct pstat *curproc)
 			continue;
 		}
 
-		if (memcmp(line, "Tgid:", 4)==0)
-		{
-			sscanf(line, "Tgid: %d", &tgid);
-			continue;
-		}
-
 		if (memcmp(line, "Uid:", 4)==0)
 		{
 			sscanf(line, "Uid: %d", &ruid);
@@ -434,7 +433,7 @@ fillproc(struct pstat *curproc)
 	curproc->gen.pid      = pid;
 	curproc->gen.ruid     = ruid;
 	curproc->gen.rgid     = rgid;
-	curproc->gen.tgid     = tgid;
+	curproc->gen.ppid     = ppid;
 	curproc->gen.nthr     = nthreads;
 	curproc->gen.state    = state;
 
@@ -476,6 +475,93 @@ fillproc(struct pstat *curproc)
 	curproc->net.rawsnd   = rawsnd;
 	curproc->net.rawrcv   = rawrcv;
 
+	/*
+	** check the state of every individual thread
+	*/
+	if (nthreads > 1)
+	{
+		DIR		*dirt;
+		struct dirent	*tent;
+		FILE		*tstat;
+		char		tpath[128];
+
+		/*
+		** open underlying task directory
+		*/
+		if ( chdir("task") == 0 )
+		{
+			dirt = opendir(".");
+
+			while ( (tent = readdir(dirt)) )
+			{
+				/*
+				** check if valid thread directory underneath
+				*/
+				if (!isdigit(tent->d_name[0]))
+					continue;
+
+				/*
+				** open stat-file of thread
+				*/
+				snprintf(tpath, sizeof tpath,
+						"%s/stat", tent->d_name);
+
+				if ( (tstat = fopen(tpath, "r")) == NULL)
+					continue;
+
+				/*
+				** read (first) line from file
+				*/
+				if (fgets(line, sizeof line, tstat) == NULL)
+				{
+					fclose(tstat);	/* failed */
+					continue;
+				}
+
+				/*
+				** fetch required info from thread
+				*/
+				cmdtail = strrchr(line, ')');
+
+				nr = sscanf(cmdtail+2, "%c", &state);
+
+				fclose(tstat);
+
+				switch (state)
+				{
+				   case 'R':
+					curproc->gen.nthrrun++;
+					break;
+				   case 'S':
+					curproc->gen.nthrslpi++;
+					break;
+				   case 'D':
+					curproc->gen.nthrslpu++;
+					break;
+				}
+			}
+
+			closedir(dirt);
+
+			chdir("..");
+		}
+	}
+	else
+	{
+		switch (state)
+		{
+		   case 'R':
+			curproc->gen.nthrrun  = 1;
+			break;
+		   case 'S':
+			curproc->gen.nthrslpi = 1;
+			break;
+		   case 'D':
+			curproc->gen.nthrslpu = 1;
+			break;
+		}
+	}
+
 	return nthreads;
 }
 
