@@ -36,14 +36,17 @@ struct pstat {
 		int	pid;		/* process identification 	*/
 		int	ruid;		/* real user  identification 	*/
 		int	rgid;		/* real group identification 	*/
-		int	tgid;		/* thread-group identification 	*/
+		int	ppid;           /* parent process identification*/
 		int	nthr;		/* number of threads in tgroup 	*/
 		char	name[PNAMLEN+1];/* process name string       	*/
 		char 	state;		/* process state ('E' = exited)	*/
 		int	excode;		/* process exit status		*/
 		time_t 	btime;		/* process start time (epoch)	*/
 		char	cmdline[CMDLEN+1];/* command-line string       	*/
-		int	ifuture[4];	/* reserved for future use	*/
+		int	nthrslpi;	/* # threads in state 'S'       */
+		int	nthrslpu;	/* # threads in state 'D'       */
+		int	nthrrun;	/* # threads in state 'R'       */
+		int	ifuture[1];     /* reserved                     */
 	} gen;
 
 	/* CPU STATISTICS						*/
