@@ -30,6 +30,9 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: photosyst.c,v $
+** Revision 1.30  2008/02/25 13:47:00  gerlof
+** Experimental code for HTTP statistics.
+**
 ** Revision 1.29  2007/08/17 09:45:44  gerlof
 ** Experimental: gather info about HTTP statistics.
 **
@@ -121,7 +124,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.29 2007/08/17 09:45:44 gerlof Exp $";
+static const char rcsid[] = "$Id: photosyst.c,v 1.30 2008/02/25 13:47:00 gerlof Exp $";
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -961,7 +964,6 @@ getwwwstat(unsigned short port, struct wwwstat *wp)
 
 	sockname.sin_family		= AF_INET;
 	sockname.sin_addr.s_addr	= htonl(INADDR_LOOPBACK);
-	sockname.sin_addr.s_addr	= htonl(0xc36ce51e);
 	sockname.sin_port		= htons(port);
 
 	if ( connect(sockfd, (struct sockaddr *) &sockname,
