@@ -61,19 +61,6 @@ struct	netstat {
 	struct udpv6_stats	udpv6;
 
 	struct tcp_stats	tcp;
-#if 0
-	count_t	tcpin;		/* TCP received packets		*/
-	count_t	tcpout;		/* TCP transmitted packets	*/
-
-	count_t	udpin;		/* UDP received packets		*/
-	count_t	udpout;		/* UDP transmitted packets	*/
-
-	count_t	ipin;		/* IP  received packets		*/
-	count_t	ipout;		/* IP  transmitted packets	*/
-	count_t	ipindel;	/* IP  local delivered packets	*/
-	count_t	ipfrw;		/* IP  forwarded packets	*/
-	count_t	cfuture[8];	/* reserved for future use	*/
-#endif
 };
 
 /************************************************************************/
@@ -192,3 +179,4 @@ struct	sstat {
 */
 void	photosyst (struct sstat *);
 void	deviatsyst(struct sstat *, struct sstat *, struct sstat *);
+void	totalsyst (char,           struct sstat *, struct sstat *);
