@@ -32,6 +32,9 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: procdbase.c,v $
+** Revision 1.7  2007/11/05 12:12:31  gerlof
+** Match processes not only on pid, but also on start time.
+**
 ** Revision 1.6  2005/10/21 09:50:19  gerlof
 ** Per-user accumulation of resource consumption.
 **
@@ -52,7 +55,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: procdbase.c,v 1.6 2005/10/21 09:50:19 gerlof Exp $";
+static const char rcsid[] = "$Id: procdbase.c,v 1.7 2007/11/05 12:12:31 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -84,7 +87,7 @@ static struct pinfo	presidue;
 ** search process database for the given PID
 */
 int
-pdb_getproc(int pid, struct pinfo **pinfopp)
+pdb_getproc(int pid, time_t btime, struct pinfo **pinfopp)
 {
 	register struct pinfo	*pp;
 
@@ -99,7 +102,7 @@ pdb_getproc(int pid, struct pinfo **pinfopp)
 		** if this is required PID, unchain it from the RESIDUE-list
 		** and return info
 		*/
-		if (pp->pstat.gen.pid == pid)
+		if (pp->pstat.gen.pid == pid && pp->pstat.gen.btime == btime)
 		{
 			if (pp->prnext)		/* if part of RESIDUE-list   */
 			{
