@@ -26,6 +26,9 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: rawlog.c,v $
+** Revision 1.22  2008/01/07 10:18:05  gerlof
+** Implement possibility to make summaries.
+**
 ** Revision 1.21  2007/08/16 12:01:25  gerlof
 ** Add support for atopsar reporting.
 **
@@ -97,7 +100,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: rawlog.c,v 1.21 2007/08/16 12:01:25 gerlof Exp $";
+static const char rcsid[] = "$Id: rawlog.c,v 1.22 2008/01/07 10:18:05 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -119,7 +122,7 @@ static const char rcsid[] = "$Id: rawlog.c,v 1.21 2007/08/16 12:01:25 gerlof Exp
 #include "photoproc.h"
 #include "photosyst.h"
 
-#define	BASEPATH	"/var/log/"  
+#define	BASEPATH	"/var/log/atop/"  
 
 /*
 ** structure which describes the raw file contents
@@ -175,8 +178,6 @@ struct rawrecord {
 	unsigned int	ifuture[6];	/* future use                   */
 };
 
-#define	RRBOOT	0x0001	/* rawrecord.flags				*/
-
 static int	getrawrec  (int, struct rawrecord *, int);
 static int	getrawsstat(int, struct sstat *, int);
 static int	getrawpstat(int, struct pstat *, int, int);
@@ -192,7 +193,7 @@ char
 rawwrite(time_t curtime, int numsecs, 
 	 struct sstat *ss, struct pstat *ps,
 	 int nlist, int npresent, int nzombie,
-	 int nexit, char bootflag)
+	 int nexit, char flag)
 {
 	static int		rawfd = -1;
 	struct rawrecord	rr;
@@ -242,7 +243,7 @@ rawwrite(time_t curtime, int numsecs,
 	rr.scomplen	= scomplen;
 	rr.pcomplen	= pcomplen;
 
-	if (bootflag)
+	if (flag&RRBOOT)
 		rr.flags |= RRBOOT;
 
 	if ( write(rawfd, &rr, sizeof rr) == -1)
@@ -400,6 +401,8 @@ rawread(unsigned int begintime, unsigned int endtime)
 	struct sstat		devsstat;
 	struct pstat		*devpstat;
 
+	struct stat		filestat;
+
 	/*
 	** variables to maintain the offsets of the raw records
 	** to be able to see previous samples again
@@ -407,7 +410,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 	off_t			*offlist;
 	unsigned int		offsize = 0;
 	unsigned int		offcur  = 0;
-	char			lastcmd = 'X';
+	char			lastcmd = 'X', flags;
 
 	time_t			timenow;
 	struct tm		*tp;
@@ -421,7 +424,7 @@ rawread(unsigned int begintime, unsigned int endtime)
 		timenow	= time(0);
 		tp	= localtime(&timenow);
 
-		snprintf(rawname, RAWNAMESZ, "%s/atop.log",
+		snprintf(rawname, RAWNAMESZ, "%s/atop_%04d%02d%02d",
 			BASEPATH, 
 			tp->tm_year+1900,
 			tp->tm_mon+1,
@@ -627,12 +630,19 @@ rawread(unsigned int begintime, unsigned int endtime)
 			*/
 			sampcnt++;
 
+			flags = rr.flags & RRBOOT;
+
+			(void) fstat(rawfd, &filestat);
+
+			if ( filestat.st_size -
+			     lseek(rawfd, (off_t)0, SEEK_CUR) <= rh.rawreclen)
+				flags |= RRLAST;
+
 			lastcmd = (vis.show_samp)(rr.curtime, rr.interval,
 						&devsstat,    devpstat,
 					 	rr.nlist,     rr.npresent,
 						rr.nzombie,   rr.nexit,
-						rr.flags&RRBOOT);
-	
+						flags);
 			free(devpstat);
 	
 			switch (lastcmd)
