@@ -41,6 +41,13 @@ struct selection {
 	regex_t	procregex;
 };
 
+/*
+** color names
+*/
+#define	COLORLOW	2
+#define	COLORMED	3
+#define	COLORHIGH	4
+
 /*
 ** list with keystrokes/flags
 */
@@ -59,7 +66,9 @@ struct selection {
 #define	MSORTDSK	'D'
 #define	MSORTMEM	'M'
 #define	MSORTNET	'N'
+#define	MSORTAUTO	'A'
 
+#define	MCOLORS		'x'
 #define	MSYSFIXED	'f'
 #define	MSYSLIMIT	'l'
 
@@ -97,7 +106,8 @@ void	showtotproc(struct pstat *, double, int, int);
 void	showcmdproc(struct pstat *, double, int, int);
 
 void	printg     (const char *, ...);
-int	prisyst(struct sstat  *, int, int, int, int, int, int, int);
-int	priproc(struct pstat  *, int, int, int, int, int, char *,
-	        struct syscap *, char, struct selection *, int, int, int);
-void	priphead(int, int, char *, char);
+int	prisyst(struct sstat  *, int, int, int, int, int, char *,
+                int, int, int);
+int	priproc(struct pstat  *, int, int, int, int, int, char, char,
+	        struct syscap *, struct selection *, int, int);
+void	priphead(int, int, char, char, char);
