@@ -29,6 +29,16 @@
 ** --------------------------------------------------------------------------
 **
 ** $Log: showlinux.c,v $
+** Revision 1.49  2008/03/06 08:38:28  gerlof
+** Register/show ppid of a process.
+**
+** Revision 1.48  2008/01/18 07:37:05  gerlof
+** Show information about the state of the individual threads
+** in the scheduling report shown with keystroke 's'.
+**
+** Revision 1.47  2008/01/07 11:34:18  gerlof
+** Correct the sort-order of network-interfaces (on busy-percentage).
+**
 ** Revision 1.46  2007/11/07 09:23:29  gerlof
 ** Modified format for avg1, avg5 and avg15 (CPL) when counters too large.
 **
@@ -183,7 +193,7 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.46 2007/11/07 09:23:29 gerlof Exp $";
+static const char rcsid[] = "$Id: showlinux.c,v 1.49 2008/03/06 08:38:28 gerlof Exp $";
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -293,11 +303,11 @@ char 	*netprocline	= "                                        "
 char 	*netproclinep	= "%5d %s %s|%s %s|%s %s|%s %s|%2llu %2llu|"
 			  "%3.0lf%% <%.12s>\n";
 /***************************************************************************/
-char	*varprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
-	                  "ST EXC  S %c%s CMD  %4d/%-4d\n";
-char 	*varproclina	= "%5d  %-8.8s %-8.8s  %s %s     %c%c   -  %c %3.0lf%% "
+char	*varprochdr	= "\n  PID  PPID USERNAME GROUP      "
+			  "STDATE    STTIME  ST EXC  S %c%s CMD  %4d/%-4d\n";
+char 	*varproclina	= "%5d %5d %-8.8s %-8.8s %s %s %c%c   -  %c %3.0lf%% "
 	                  "%.14s\n";
-char 	*varprocline	= " %-8.8s %-8.8s  %s %s     %c%c %3d  E %3.0lf%% "
+char 	*varprocline	= "%5d %-8.8s %-8.8s %s %s %c%c %3d  E %3.0lf%% "
 			  "<%.12s>\n";
 /***************************************************************************/
 char	*cmdprochdr	= "\n  PID %c%s COMMAND-LINE                           "
@@ -305,12 +315,12 @@ char	*cmdprochdr	= "\n  PID %c%s COMMAND-LINE                           "
 char 	*cmdproclina	= "%5d %3.0lf%% %s\n";
 char 	*cmdprocline	= "%3.0lf%% <%.66s>\n";
 /***************************************************************************/
-char	*schprochdr	= "\n  PID  TGID  THR  POLICY NICE PRIO RTPRIO "
-			  "CURCPU ST EXC   S %c%s CMD  %4d/%-4d\n";
-char 	*schproclina	= "%5d %5d %4d  %-6s %4d %4d %6d %6d %-2s   -   "
+char	*schprochdr	= "\n  PID TRUN TSLPI TSLPU  POLI NICE PRI RTPR "
+			  "CURCPU ST EXC  S %c%s CMD  %4d/%-4d\n";
+char 	*schproclina	= "%5d %4d %5d %5d  %-4s %4d %3d %4d %6d %-2s   -  "
 			  "%c %3.0lf%% %.14s\n";
-char 	*schprocline	= "     -    0       -    -    -      -      - "
-			  "%c%c %3d   E %3.0lf%% <%.12s>\n";
+char 	*schprocline	= "    0     0     0     -    -   -    -      - "
+			  "%c%c %3d  E %3.0lf%% <%.12s>\n";
 /***************************************************************************/
 char	*totuserhdr	= "\nNPROCS  SYSCPU  USRCPU  VSIZE  RSIZE  RDDSK "
 			  "WRDSK RNET SNET %c%s USER %4d/%-4d\n";
@@ -1013,6 +1023,7 @@ showvarproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		*/
 		printg(varproclina,
 		  	curstat->gen.pid,
+		  	curstat->gen.ppid,
 		  	username,
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
@@ -1045,6 +1056,7 @@ showvarproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		}
 
 		printg(varprocline,
+			curstat->gen.ppid,
 		  	username,
 		  	groupname,
 			convdate(curstat->gen.btime, format1),
@@ -1110,10 +1122,10 @@ showschproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 	switch (curstat->cpu.policy)
 	{
 	   case SCHED_NORMAL:
-		pol = "normal";
+		pol = "norm";
 		break;
 	   case SCHED_RR:
-		pol = "roundr";
+		pol = "rr  ";
 		break;
 	   case SCHED_FIFO:
 		pol = "fifo";
@@ -1132,8 +1144,9 @@ showschproc(struct pstat *curstat, double perc, int nsecs, int avgval)
 		*/
 		printg(schproclina,
 		  	curstat->gen.pid,
-		  	curstat->gen.tgid,
-		  	curstat->gen.nthr,
+		  	curstat->gen.nthrrun,
+		  	curstat->gen.nthrslpi,
+		  	curstat->gen.nthrslpu,
 		  	pol,
 		  	curstat->cpu.nice,
 		  	curstat->cpu.prio,
@@ -2073,14 +2086,63 @@ diskcompar(const void *a, const void *b)
 int
 intfcompar(const void *a, const void *b)
 {
-	register count_t apack = ((struct perintf *)a)->rpack +
-			         ((struct perintf *)a)->spack;
-	register count_t bpack = ((struct perintf *)b)->rpack +
-			         ((struct perintf *)b)->spack;
+	register count_t afactor=0, bfactor=0;
 
-	if (apack < bpack) return  1;
-	if (apack > bpack) return -1;
-	                   return  0;
+	count_t aspeed         = ((struct perintf *)a)->speed;
+	count_t bspeed         = ((struct perintf *)b)->speed;
+	char    aduplex        = ((struct perintf *)a)->duplex;
+	char    bduplex        = ((struct perintf *)b)->duplex;
+	count_t arbyte         = ((struct perintf *)a)->rbyte;
+	count_t brbyte         = ((struct perintf *)b)->rbyte;
+	count_t asbyte         = ((struct perintf *)a)->sbyte;
+	count_t bsbyte         = ((struct perintf *)b)->sbyte;
+
+
+	/*
+	** if speed of first interface known, calculate busy factor
+	*/
+	if (aspeed)
+	{
+		if (aduplex)
+			afactor = (arbyte > asbyte ? arbyte : asbyte) /
+								(aspeed / 10);
+		else
+			afactor = (arbyte + asbyte) /		(aspeed / 10);
+	}
+
+	/*
+	** if speed of second interface known, calculate busy factor
+	*/
+	if (bspeed)
+	{
+		if (bduplex)
+			bfactor = (brbyte > bsbyte ? brbyte : bsbyte) /
+								(bspeed / 10);
+		else
+			bfactor = (brbyte + bsbyte) /		(bspeed / 10);
+	}
+
+	/*
+	** compare interfaces
+	*/
+	if (aspeed && bspeed)
+	{
+		if (afactor < bfactor)	return  1;
+		if (afactor > bfactor)	return -1;
+					return  0;
+	}
+
+	if (!aspeed && !bspeed)
+	{
+		if ((arbyte + asbyte) < (brbyte + bsbyte))	return  1;
+		if ((arbyte + asbyte) > (brbyte + bsbyte))	return -1;
+								return  0;
+	}
+
+	if (aspeed)
+		return -1;
+	else
+		return  1;
 }
 
 int
